"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts = __importStar(require("typescript"));
const path_1 = __importDefault(require("path"));
const TypeScriptIssueFactory_1 = require("../issue/TypeScriptIssueFactory");
const ControlledWatchCompilerHost_1 = require("./ControlledWatchCompilerHost");
const TypeScriptVueExtension_1 = require("../extension/vue/TypeScriptVueExtension");
const ControlledWatchSolutionBuilderHost_1 = require("./ControlledWatchSolutionBuilderHost");
const ControlledTypeScriptSystem_1 = require("./ControlledTypeScriptSystem");
const TypeScriptConfigurationParser_1 = require("./TypeScriptConfigurationParser");
function createTypeScriptReporter(configuration) {
    const extensions = [];
    let system;
    let parsedConfiguration;
    let watchCompilerHost;
    let watchSolutionBuilderHost;
    let watchProgram;
    let solutionBuilder;
    const diagnosticsPerProject = new Map();
    if (configuration.extensions.vue.enabled) {
        extensions.push(TypeScriptVueExtension_1.createTypeScriptVueExtension(configuration.extensions.vue));
    }
    function getProjectNameOfBuilderProgram(builderProgram) {
        return builderProgram.getProgram().getCompilerOptions().configFilePath;
    }
    function getDiagnosticsOfBuilderProgram(builderProgram) {
        const diagnostics = [];
        if (configuration.diagnosticOptions.syntactic) {
            diagnostics.push(...builderProgram.getSyntacticDiagnostics());
        }
        if (configuration.diagnosticOptions.semantic) {
            diagnostics.push(...builderProgram.getSemanticDiagnostics());
        }
        if (configuration.diagnosticOptions.declaration) {
            diagnostics.push(...builderProgram.getDeclarationDiagnostics());
        }
        if (configuration.diagnosticOptions.global) {
            diagnostics.push(...builderProgram.getGlobalDiagnostics());
        }
        return diagnostics;
    }
    return {
        getReport: ({ changedFiles = [], deletedFiles = [] }) => __awaiter(this, void 0, void 0, function* () {
            if (!system) {
                system = ControlledTypeScriptSystem_1.createControlledTypeScriptSystem(configuration.mode);
            }
            // clear cache to be ready for next iteration and to free memory
            system.clearCache();
            if ([...changedFiles, ...deletedFiles]
                .map((affectedFile) => path_1.default.normalize(affectedFile))
                .includes(path_1.default.normalize(configuration.tsconfig))) {
                // we need to re-create programs
                parsedConfiguration = undefined;
                watchCompilerHost = undefined;
                watchSolutionBuilderHost = undefined;
                watchProgram = undefined;
                solutionBuilder = undefined;
                diagnosticsPerProject.clear();
            }
            if (!parsedConfiguration) {
                const parseConfigurationDiagnostics = [];
                parsedConfiguration = TypeScriptConfigurationParser_1.parseTypeScriptConfiguration(configuration.tsconfig, configuration.context, configuration.compilerOptions, Object.assign(Object.assign({}, system), { onUnRecoverableConfigFileDiagnostic: (diagnostic) => {
                        parseConfigurationDiagnostics.push(diagnostic);
                    } }));
                if (parsedConfiguration.errors) {
                    parseConfigurationDiagnostics.push(...parsedConfiguration.errors);
                }
                // report configuration diagnostics and exit
                if (parseConfigurationDiagnostics.length) {
                    parsedConfiguration = undefined;
                    let issues = TypeScriptIssueFactory_1.createIssuesFromTsDiagnostics(parseConfigurationDiagnostics);
                    issues.forEach((issue) => {
                        if (!issue.file) {
                            issue.file = configuration.tsconfig;
                        }
                    });
                    extensions.forEach((extension) => {
                        if (extension.extendIssues) {
                            issues = extension.extendIssues(issues);
                        }
                    });
                    return issues;
                }
            }
            if (configuration.build) {
                // solution builder case
                // ensure watch solution builder host exists
                if (!watchSolutionBuilderHost) {
                    watchSolutionBuilderHost = ControlledWatchSolutionBuilderHost_1.createControlledWatchSolutionBuilderHost(parsedConfiguration, system, ts.createSemanticDiagnosticsBuilderProgram, undefined, undefined, undefined, undefined, (builderProgram) => {
                        const projectName = getProjectNameOfBuilderProgram(builderProgram);
                        const diagnostics = getDiagnosticsOfBuilderProgram(builderProgram);
                        // update diagnostics
                        diagnosticsPerProject.set(projectName, diagnostics);
                    }, extensions);
                    solutionBuilder = undefined;
                }
                // ensure solution builder exists
                if (!solutionBuilder) {
                    solutionBuilder = ts.createSolutionBuilderWithWatch(watchSolutionBuilderHost, [configuration.tsconfig], {});
                    solutionBuilder.build();
                }
            }
            else {
                // watch compiler case
                // ensure watch compiler host exists
                if (!watchCompilerHost) {
                    watchCompilerHost = ControlledWatchCompilerHost_1.createControlledWatchCompilerHost(parsedConfiguration, system, ts.createSemanticDiagnosticsBuilderProgram, undefined, undefined, (builderProgram) => {
                        const projectName = getProjectNameOfBuilderProgram(builderProgram);
                        const diagnostics = getDiagnosticsOfBuilderProgram(builderProgram);
                        // update diagnostics
                        diagnosticsPerProject.set(projectName, diagnostics);
                    }, extensions);
                    watchProgram = undefined;
                }
                // ensure watch program exists
                if (!watchProgram) {
                    watchProgram = ts.createWatchProgram(watchCompilerHost);
                }
            }
            changedFiles.forEach((changedFile) => {
                if (system) {
                    system.invokeFileChanged(changedFile);
                }
            });
            deletedFiles.forEach((removedFile) => {
                if (system) {
                    system.invokeFileDeleted(removedFile);
                }
            });
            // wait for all queued events to be processed
            yield system.waitForQueued();
            // aggregate all diagnostics and map them to issues
            const diagnostics = [];
            diagnosticsPerProject.forEach((projectDiagnostics) => {
                diagnostics.push(...projectDiagnostics);
            });
            let issues = TypeScriptIssueFactory_1.createIssuesFromTsDiagnostics(diagnostics);
            extensions.forEach((extension) => {
                if (extension.extendIssues) {
                    issues = extension.extendIssues(issues);
                }
            });
            return issues;
        }),
    };
}
exports.createTypeScriptReporter = createTypeScriptReporter;
